 /**************************************************************************
 *   Copyright (C) 2009 by Silicium628 (France)                            *
 *   http://silicium628.chez-alice.fr/                                     *
 *   http://www.silicium628.fr/                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation;                                         *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY;                                             *
 *   See the GNU General Public License for more details.                  *
 ***************************************************************************/

#include "vecteur2d_v2.h"
#include <math.h>

#include "complexe.cpp"



double echelle = 94.4;

Vecteur2D::Vecteur2D()
{
    dx=0.0;
    dy=0.0;
}


double Vecteur2D::get_dx(void) { return dx; }
double Vecteur2D::get_dy(void) { return dy; }

void Vecteur2D::set_dx(double dx_i) { dx = dx_i; }
void Vecteur2D::set_dy(double dy_i) { dy = dy_i; }

void Vecteur2D::set_offsetX(double offsetX_i){ offsetX = offsetX_i; };
void Vecteur2D::set_offsetY(double offsetY_i){ offsetY = offsetY_i; };

void Vecteur2D::multi_lambda(double lambda) // produit par un réel
{ 
    dx *= lambda;
    dy *= lambda;
}

Vecteur2D Vecteur2D::operator+ (Vecteur2D &Vi)
{
    Vecteur2D Vr;
    Vr.dx=this->dx+Vi.dx;
    Vr.dy=this->dy+Vi.dy;
    return Vr;
}

Vecteur2D Vecteur2D::operator- (Vecteur2D &Vi)
{
    Vecteur2D Vr;
    Vr.dx=this->dx-Vi.dx;
    Vr.dy=this->dy-Vi.dy;
    return Vr;
}


void Vecteur2D::dessiner(double x_i, double y_i, QGraphicsItemGroup *groupe_i, QPen pen_i, int fle, QString ltr, int ecr, int crl, int prj)
{
    double module;
    double sin_alpha, cos_alpha;
    double dx2, dy2; // vecteur orthogonal

    QPointF A,B,C,D,E; // A="origine" B="extrémité" C et D et E = base du triangle de la flèche

    A.setX(offsetX+echelle*x_i);
    A.setY(offsetY-echelle*y_i);
    B.setX(A.x()+echelle*dx);
    B.setY(A.y()-echelle*dy);

//dessin de la ligne du vecteur
    ligne_i = new QGraphicsLineItem(A.x(),A.y(),B.x(),B.y());  ligne_i->setPen(pen_i);
    groupe_i->addToGroup(ligne_i);


//calcul d'un vecteur perpendiculaire
    C.setX(A.x()+0.93*echelle*dx); // un point situé sur le vecteur, à 93% de sa longueur et entre D et E
    C.setY(A.y()-0.93*echelle*dy);

    module = sqrt(dx*dx + dy*dy);


    if((module !=0) && (fle==1))
    {
        cos_alpha=dx/module;
        sin_alpha=dy/module;

        dx2 = -0.04 * module * sin_alpha; // "petit" vecteur orthoganal de taille proportionnelle au module
        dy2 =  0.04 * module * cos_alpha;

//   dx2 = - 0.1 * sin_alpha; // "petit" vecteur orthoganal de taille fixe
//   dy2= 0.1 * cos_alpha;
//   A.setX(offsetX+x_i);
//   A.setY(offsetY+y_i);

        D.setX(C.x()+echelle*dx2);
        D.setY(C.y()-echelle*dy2);
        E.setX(C.x()-echelle*dx2);
        E.setY(C.y()+echelle*dy2);

//dessin du triangle de la flèche
        QGraphicsPolygon1 = new QGraphicsPolygonItem;

        QPolygonF polygone2;
        polygone2 << B << D << E;

        QBrush brush1;
        brush1.setColor(pen_i.color());
        brush1.setStyle(Qt::SolidPattern);
        QGraphicsPolygon1->setPolygon(polygone2);
        QGraphicsPolygon1->setBrush(brush1);
        groupe_i->addToGroup(QGraphicsPolygon1);

        if(ecr == 1)
        {
            texte1 = new QGraphicsSimpleTextItem;
            texte1->setText(ltr.mid(0,1));
            texte1->setPos(A.x()+10,A.y()+10);
            groupe_i->addToGroup(texte1);

            texte1 = new QGraphicsSimpleTextItem;
            texte1->setText(ltr.mid(1,1));
            texte1->setPos(B.x()+10,B.y()+10);
            groupe_i->addToGroup(texte1);
        }

        if(ecr == 1)
        {
// tracé de points aux extrémités
            QBrush brush2;
            brush2.setStyle(Qt::SolidPattern);
            brush2.setColor(QColor::fromRgb(0, 0, 0, 255));

            ellipse1 = new QGraphicsEllipseItem;
            ellipse1->setBrush(brush2);
            ellipse1->setRect(A.x()-3,A.y()-3,6,6);
            groupe_i->addToGroup(ellipse1);

            ellipse1 = new QGraphicsEllipseItem;
            ellipse1->setBrush(brush2);
            ellipse1->setRect(B.x()-2,B.y()-2,4,4);
            groupe_i->addToGroup(ellipse1);
        }

// tracer projections sur les axes x et y
        if(prj == 1)
        {
           ligne_i = new QGraphicsLineItem(B.x(),A.y(),B.x(),B.y()); // ligne V
           groupe_i->addToGroup(ligne_i);

           ligne_i = new QGraphicsLineItem(A.x(),B.y(),B.x(),B.y()); // ligne H
           groupe_i->addToGroup(ligne_i);

        }

        if(crl == 1)
        {
//tracé d'un cercle de rayon = module
            ellipse1 = new QGraphicsEllipseItem;
            //ellipse1->setPen(pen_i);
            double rayon=module*echelle;
            ellipse1->setRect(A.x()-rayon,A.y()-rayon,2*rayon, 2*rayon);
            groupe_i->addToGroup(ellipse1);
        }

    }
}




